# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from glance_tempest_plugin.tests.rbac.v2 import base as rbac_base

CONF = config.CONF


class ImagesProjectMemberTests(rbac_base.ImageV2RbacImageTest,
                               rbac_base.ImageV2RbacTemplate):

    credentials = ['project_member', 'project_admin', 'system_admin',
                   'admin', 'project_alt_admin', 'project_alt_member']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.alt_project_images_client = (
            cls.os_project_alt_member.image_client_v2)

    @decorators.idempotent_id('a71e7caf-2403-4fed-a4bf-9717949ecde2')
    def test_create_image(self):
        image = self.do_request('create_image', expected_status=201,
                                **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        image = self.do_request('create_image', expected_status=201,
                                **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        image = self.do_request('create_image', expected_status=201,
                                **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        self.do_request('create_image', expected_status=exceptions.Forbidden,
                        **self.image(visibility='public'))

    @decorators.idempotent_id('2adf7202-7fc9-4a6e-b6dd-fb3d40365ccb')
    def test_get_image(self):
        # Ensure users can get private images owned by their project.
        project_id = self.persona.credentials.project_id
        project_member = self.setup_user_client(project_id=project_id)
        image = project_member.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])

        image = project_member.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', expected_status=exceptions.NotFound,
                        image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', expected_status=exceptions.NotFound,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', expected_status=exceptions.NotFound,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        project_id = self.persona.credentials.project_id
        self.admin_client.image_member_client_v2.create_image_member(
            image['id'], member=project_id)
        self.addCleanup(
            self.admin_client.image_member_client_v2.delete_image_member,
            image['id'], project_id)

        self.do_request('show_image', image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])

        self.do_request('show_image', expected_status=exceptions.NotFound,
                        image_id=data_utils.rand_uuid())

    @decorators.idempotent_id('259d5578-410e-4b0f-bb2d-cb5b057bc696')
    def test_list_images(self):
        project_id = self.persona.credentials.project_id
        project_member = self.setup_user_client(project_id=project_id)
        project_client = self.setup_user_client()

        # Create a private image in the project
        private_image_in_project = project_member.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        private_image_in_project['id'])

        # Create a private image without an owner
        private_image_no_owner = self.os_admin.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        private_image_no_owner['id'])

        # Create a private image in another project
        private_image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        private_image['id'])

        # Create a public image
        public_image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image,
                        public_image['id'])

        resp = self.do_request('list_images', expected_status=200)
        image_ids = set(image['id'] for image in resp['images'])

        self.assertNotIn(private_image_no_owner['id'], image_ids)
        self.assertNotIn(private_image['id'], image_ids)
        self.assertIn(public_image['id'], image_ids)
        self.assertIn(private_image_in_project['id'], image_ids)

        # Create a shared image in another project
        shared_image_1 = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        shared_image_2 = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        shared_image_1['id'])
        self.addCleanup(self.admin_images_client.delete_image,
                        shared_image_2['id'])

        # Share the image from the other project with the user's project
        project_id = self.persona.credentials.project_id
        self.admin_client.image_member_client_v2.create_image_member(
            shared_image_1['id'], member=project_id)
        self.addCleanup(
            self.admin_client.image_member_client_v2.delete_image_member,
            shared_image_1['id'], project_id)

        # List images and assert the shared image is not in the list of images
        # because it hasn't been accepted, yet.
        resp = self.do_request('list_images', expected_status=200)
        image_ids = set(image['id'] for image in resp['images'])
        self.assertNotIn(shared_image_1['id'], image_ids)
        self.assertNotIn(shared_image_2['id'], image_ids)

        # Accept the image and ensure it's returned in the list of images
        project_member.image_member_client_v2.update_image_member(
            shared_image_1['id'], project_id, status='accepted')
        resp = self.do_request('list_images', expected_status=200)
        image_ids = set(image['id'] for image in resp['images'])
        self.assertIn(shared_image_1['id'], image_ids)
        self.assertNotIn(shared_image_2['id'], image_ids)

    @decorators.idempotent_id('13f8949b-3419-4a4a-bd0b-71fa711206fd')
    def test_update_image(self):
        image = self.client.create_image(**self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)

        image = self.client.create_image(**self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=exceptions.NotFound,
                        image_id=image['id'], patch=patch_body,
                        client=self.alt_project_images_client)

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=exceptions.NotFound,
                        image_id=image['id'], patch=patch_body)

        image = self.client.create_image(**self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=exceptions.NotFound,
                        image_id=image['id'], patch=patch_body)

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'], patch=patch_body,
                        client=self.alt_project_images_client)

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'], patch=patch_body,
                        client=self.alt_project_images_client)

    @decorators.idempotent_id('bd5845dc-d96b-4d83-a8da-7978bd91ddc1')
    def test_upload_image(self):
        file_contents = data_utils.random_bytes()
        image_data = io.BytesIO(file_contents)

        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        # upload file for private image - pass
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file', expected_status=204,
                        image_id=image['id'], data=image_data)

        # upload file for shared image - pass
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file', expected_status=204,
                        image_id=image['id'], data=image_data)

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], data=image_data,
                        client=self.alt_project_images_client)

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], data=image_data)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], data=image_data)

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'], data=image_data,
                        client=self.alt_project_images_client)

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'], data=image_data,
                        client=self.alt_project_images_client)

    @decorators.idempotent_id('3dfa6f70-f6fe-4ed5-96eb-5f4634064aa3')
    def test_download_image(self):
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file', expected_status=exceptions.NotFound,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file', expected_status=exceptions.NotFound,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image_file', expected_status=exceptions.NotFound,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])

    @decorators.idempotent_id('326d267a-de9d-4217-aadc-a0f2b5993537')
    def test_delete_image(self):

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.do_request('delete_image', expected_status=exceptions.NotFound,
                        image_id=image['id'],
                        client=self.alt_project_images_client)
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.do_request('delete_image', expected_status=exceptions.NotFound,
                        image_id=image['id'])
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        image = self.client.create_image(
            **self.image(visibility='private'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.do_request('delete_image', expected_status=exceptions.NotFound,
                        image_id=image['id'])
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        project_id = self.persona.credentials.project_id
        project_member = self.setup_user_client(project_id=project_id)
        image = project_member.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='community'))
        self.do_request('delete_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'])
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        image = self.client.create_image(
            **self.image(visibility='community'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.do_request('delete_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'],
                        client=self.alt_project_images_client)
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

    @decorators.idempotent_id('395579c9-92bb-40a9-a8b6-9daaa20ae610')
    def test_add_image_member(self):
        member_client = self.setup_user_client()
        member_project_id = member_client.credentials.project_id
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        # As users with authorization on the project that owns the image, we
        # should be able to share that image with other projects.
        self.do_request('create_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member=member_project_id)

    @decorators.idempotent_id('67510e3f-57cd-4a76-9e96-577e49229677')
    def test_get_image_member(self):
        member_client = self.setup_user_client()
        member_project_id = member_client.credentials.project_id
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)
        self.do_request('show_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member_id=member_project_id)

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)
        # The user can't show the members for this image because they can't get
        # the image or pass the get_image policy, which is processed before
        # fetching the members.
        self.do_request('show_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], member_id=member_project_id)

    @decorators.idempotent_id('64ec16c4-3b8d-464d-88fb-f274241b1302')
    def test_list_image_members(self):
        # Stash the project_id for this persona.
        project_id = self.persona.credentials.project_id

        other_member_client = self.setup_user_client()
        other_member_project_id = other_member_client.credentials.project_id

        # Create an image as an other user in a separate project and share that
        # image with this user and the other member.
        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        for p_id in [project_id, other_member_project_id]:
            project_client.image_member_client_v2.create_image_member(
                image['id'], member=p_id)
        resp = self.do_request('list_image_members',
                               client=self.persona.image_member_client_v2,
                               expected_status=200, image_id=image['id'])
        members = set(m['member_id'] for m in resp['members'])
        # Make sure this user (persona) can't view members of an image other
        # than themselves.
        self.assertIn(project_id, members)
        self.assertNotIn(other_member_project_id, members)

    @decorators.idempotent_id('12636be0-6188-4003-8824-de4f89e3c745')
    def test_update_image_member(self):
        # Create a new user with authorization on another project.
        other_project_client = self.setup_user_client()

        # Create a shared image in the other project and share the image with
        # the persona user's project.
        image = other_project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        other_project_client.image_member_client_v2.create_image_member(
            image['id'], member=self.persona.credentials.project_id)

        # Make sure the persona users can accept the image.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member_id=self.persona.credentials.project_id,
                        status='accepted')

        # Make sure the persona users can reject the image.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member_id=self.persona.credentials.project_id,
                        status='rejected')

        # Create a new user with authorization on another project to act as a
        # different member.
        member_client = self.setup_user_client()
        member_project_id = member_client.credentials.project_id

        # Create another image with the first project_client to share with the
        # new member user.
        image = other_project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        other_project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)

        # Make sure the persona user can't accept images for other projects
        # they are not a member of.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], member_id=member_project_id,
                        status='accepted')

        # Make sure the persona user can't reject images for other projects
        # they are not a member of.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], member_id=member_project_id,
                        status='rejected')

    @decorators.idempotent_id('bcfda1fa-ea65-47ce-8434-a64d85512fcf')
    def test_delete_image_member(self):
        # Create a new user with authorization on a separate project.
        member_client = self.setup_user_client()
        member_project_id = member_client.credentials.project_id

        # Create an image in the persona user's project and share that image
        # with the new member user.
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)

        # Make sure the persona user can delete image members from images they
        # own.
        self.do_request('delete_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=204,
                        image_id=image['id'], member_id=member_project_id)

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)

        # Make sure the persona user can't delete image members from images
        # outside their project.
        self.do_request('delete_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], member_id=member_project_id)

    @decorators.idempotent_id('abf9fe8b-ada3-4509-b15b-76d04e58f4e8')
    def test_deactivate_image(self):
        # Create a new user with authorization on the persona user's project.
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        file_contents = data_utils.random_bytes()
        image_data = io.BytesIO(file_contents)

        # Create a private image in the persona user's project and make sure
        # the persona user can deactivate it.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        self.do_request('deactivate_image', expected_status=204,
                        image_id=image['id'])
        resp = self.client.show_image(image_id=image['id'])
        self.assertTrue(resp['status'] == 'deactivated')

        # Create a shared image in the persona user's project and make sure the
        # persona user can deactivate it.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        self.do_request('deactivate_image', expected_status=204,
                        image_id=image['id'])
        resp = self.client.show_image(image_id=image['id'])
        self.assertTrue(resp['status'] == 'deactivated')

        # Create a new user with authorization on a separate project.
        project_client = self.setup_user_client()

        # Create a private image in that new project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        # The user can't deactivate this image because they can't find it.
        self.do_request('deactivate_image',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'])

        # Create a shared image in the new project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        # The user can't deactivate this image because they can't find it.
        self.do_request('deactivate_image',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        # Project users can't deactivate community images, only administrators
        # should be able to do this.
        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.do_request('deactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        # Project users can't deactivate public images, only administrators
        # should be able to do this.
        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.do_request('deactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

    @decorators.idempotent_id('58558447-8618-4dbe-97ce-bfc39b3743e7')
    def test_reactivate_image(self):
        # Create a new user with authorization on the persona user's project.
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        file_contents = data_utils.random_bytes()
        image_data = io.BytesIO(file_contents)

        # Create a private image within the persona user's project and make
        # sure we can reactivate it.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])
        resp = self.client.show_image(image_id=image['id'])
        self.assertTrue(resp['status'] == 'deactivated')
        self.do_request('reactivate_image', expected_status=204,
                        image_id=image['id'])
        resp = self.client.show_image(image_id=image['id'])
        self.assertTrue(resp['status'] == 'active')

        # Create a shared image within the persona user's project and make sure
        # we can reactivate it.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])
        resp = self.client.show_image(image_id=image['id'])
        self.assertTrue(resp['status'] == 'deactivated')
        self.do_request('reactivate_image', expected_status=204,
                        image_id=image['id'])
        resp = self.client.show_image(image_id=image['id'])
        self.assertTrue(resp['status'] == 'active')

        # Create a new user with authorization on a separate project.
        project_client = self.setup_user_client()

        # Create a private image in the separate project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])
        # The user can't reactivate this image because they can't find it.
        self.do_request('reactivate_image',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])
        # The user can't reactivate this image because they can't find it.
        self.do_request('reactivate_image',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'])

        # Only administrators can reactivate community images.
        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.admin_images_client.deactivate_image(image['id'])
        self.do_request('reactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        # Only administrators can reactivate public images.
        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.admin_images_client.deactivate_image(image['id'])
        self.do_request('reactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'],
                        client=self.alt_project_images_client)


class NamespacesProjectMemberTests(rbac_base.MetadefV2RbacNamespaceTest,
                                   rbac_base.MetadefV2RbacNamespaceTemplate):

    credentials = ['project_member', 'project_alt_member',
                   'project_admin', 'project_alt_admin', 'primary']

    def test_get_namespace(self):

        def assertGetNamespace(actual_ns, owner, client):
            expected_status = 200
            if (actual_ns['visibility'] == 'private' and
                    actual_ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('show_namespace',
                            expected_status=expected_status,
                            client=client,
                            namespace=actual_ns['namespace'])

        actual_namespaces = self.create_namespaces()

        # Get namespace - member role from 'project' can access all
        # namespaces of it's own & only public namespace of 'alt_project'
        for actual_ns in actual_namespaces:
            assertGetNamespace(actual_ns, self.project_id,
                               self.namespace_client)

        # Get namespace - member role from 'alt_project' can access all
        # namespaces of it's own & only public namespace of 'project'
        for actual_ns in actual_namespaces:
            assertGetNamespace(actual_ns, self.alt_project_id,
                               self.alt_namespace_client)

    def test_list_namespaces(self):
        actual_namespaces = self.create_namespaces()

        # List namespace - member role from 'project' can access all
        # namespaces of it's own & only public namespace of 'alt_project'
        resp = self.do_request('list_namespaces',
                               client=self.namespace_client,
                               expected_status=200)
        self.assertListNamespaces(actual_namespaces, resp, self.project_id)

        # List namespace - member role from 'alt_project' can access all
        # namespaces of it's own & only public namespace of 'project'
        resp = self.do_request('list_namespaces',
                               client=self.alt_namespace_client,
                               expected_status=200)
        self.assertListNamespaces(actual_namespaces, resp, self.alt_project_id)

    def test_update_namespace(self):
        actual_namespaces = self.create_namespaces()

        def assertUpdateNamespace(actual_ns, owner, client):
            expected_status = exceptions.Forbidden
            if not (actual_ns['visibility'] == 'public' or
                    actual_ns['owner'] == owner):
                expected_status = exceptions.NotFound

            self.do_request('update_namespace',
                            expected_status=expected_status,
                            client=client,
                            description=data_utils.arbitrary_string(),
                            namespace=actual_ns['namespace'])

        # Check member role of 'project' is forbidden to update namespace
        for actual_ns in actual_namespaces:
            assertUpdateNamespace(actual_ns, self.project_id,
                                  self.namespace_client)

        # Check member role of 'alt_project' is forbidden to update namespace
        for actual_ns in actual_namespaces:
            assertUpdateNamespace(actual_ns, self.alt_project_id,
                                  self.alt_namespace_client)

    def test_create_namespace(self):
        # Check non-admin role of 'project' not allowed to create namespace
        self.do_request('create_namespace',
                        expected_status=exceptions.Forbidden,
                        client=self.namespace_client,
                        namespace=data_utils.arbitrary_string())

        # Check non-admin role of 'alt_project' not allowed to create namespace
        self.do_request('create_namespace',
                        expected_status=exceptions.Forbidden,
                        client=self.alt_namespace_client,
                        namespace=data_utils.arbitrary_string())

    def test_delete_namespace(self):
        actual_namespaces = self.create_namespaces()

        def assertDeleteNamespace(actual_ns, owner, client):
            expected_status = exceptions.Forbidden
            if not (actual_ns['visibility'] == 'public' or
                    actual_ns['owner'] == owner):
                expected_status = exceptions.NotFound

            self.do_request('delete_namespace',
                            expected_status=expected_status,
                            client=client,
                            namespace=actual_ns['namespace'])

        # Check member role of 'project' is forbidden to delete namespace
        for actual_ns in actual_namespaces:
            assertDeleteNamespace(actual_ns, self.project_id,
                                  self.namespace_client)

        # Check member role of 'alt_project' is forbidden to delete namespace
        for actual_ns in actual_namespaces:
            assertDeleteNamespace(actual_ns, self.alt_project_id,
                                  self.alt_namespace_client)

        # Verify the namespaces are not deleted
        for actual_ns in actual_namespaces:
            self.do_request('show_namespace',
                            expected_status=200,
                            client=self.admin_namespace_client,
                            namespace=actual_ns['namespace'])


class RSTypesProjectMemberTests(rbac_base.MetadefV2RbacResourceTypeTest,
                                rbac_base.MetadefV2RbacResourceTypeTemplate):

    credentials = ['project_member', 'project_alt_member', 'project_admin',
                   'project_alt_admin', 'primary']

    def test_create_resource_type(self):
        namespaces = self.create_namespaces()

        def assertRSTypeCreate(namespace, owner, client):
            rs_type_name = "rs_type_of_%s" % (namespace['namespace'])
            expected_status = exceptions.Forbidden
            if (namespace['visibility'] != 'public' and
                    namespace['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('create_resource_type_association',
                            expected_status=expected_status,
                            namespace_id=namespace['namespace'],
                            name=rs_type_name,
                            client=client)

        # Make sure non admin role of 'project' forbidden to
        # create resource types
        for namespace in namespaces:
            assertRSTypeCreate(namespace, self.project_id,
                               self.resource_types_client)

    def test_get_resource_type(self):
        ns_rs_types = self.create_resource_types()

        def assertRSTypeGet(actual_rs_type, client, owner):
            ns = actual_rs_type['namespace']
            expected_status = 200
            if (ns['visibility'] != 'public' and ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('list_resource_type_association',
                            expected_status=expected_status,
                            namespace_id=ns['namespace'],
                            client=client)

        # Get resource type - member role from 'project' can access all
        # resource types of it's own & only resource types having public
        # namespace of 'alt_project'
        for rs_type in ns_rs_types:
            assertRSTypeGet(rs_type, self.resource_types_client,
                            self.project_id)

    def test_list_resource_types(self):
        ns_rs_types = self.create_resource_types()

        # list resource types - with member role of 'project'
        resp = self.do_request('list_resource_types',
                               expected_status=200,
                               client=self.resource_types_client)

        # Verify that member role of 'project' will be able to view available
        # resource types
        self.assertRSTypeList(ns_rs_types, resp)

        # list resource types -  with member role of 'alt_project'
        resp = self.do_request('list_resource_types',
                               expected_status=200,
                               client=self.resource_types_client)

    def test_delete_resource_type(self):
        ns_rs_types = self.create_resource_types()

        def assertRSTypeDelete(actual_rs_type, client, owner):
            ns = actual_rs_type['namespace']
            expected_status = exceptions.Forbidden
            if (ns['visibility'] != 'public' and ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request(
                'delete_resource_type_association',
                expected_status=expected_status,
                namespace_id=ns['namespace'],
                resource_name=actual_rs_type['resource_type']['name'],
                client=client)

        # Make sure non admin role of 'project' not allowed to
        # delete resource types
        for rs_type in ns_rs_types:
            assertRSTypeDelete(rs_type, self.resource_types_client,
                               self.project_id)


class ObjectsProjectMemberTests(rbac_base.MetadefV2RbacObjectsTest,
                                rbac_base.MetadefV2RbacObjectsTemplate):

    credentials = ['project_member', 'project_alt_member', 'project_admin',
                   'project_alt_admin', 'primary']

    def test_create_object(self):
        namespaces = self.create_namespaces()

        def assertCreateObjects(namespace, owner, client):
            object_name = "object_of_%s" % (namespace['namespace'])
            expected_status = exceptions.Forbidden
            if (namespace['visibility'] == 'private' and
                    namespace['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('create_namespace_object',
                            expected_status=expected_status,
                            namespace=namespace['namespace'],
                            name=object_name,
                            client=client)

        # Make sure non admin role of 'project' forbidden to
        # create objects
        for namespace in namespaces:
            assertCreateObjects(namespace, self.project_id,
                                self.objects_client)

    def test_get_object(self):

        def assertObjectGet(actual_obj, owner, client):
            ns = actual_obj['namespace']
            expected_status = 200
            if (ns['visibility'] == 'private' and
                    ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('show_namespace_object',
                            expected_status=expected_status,
                            namespace=actual_obj['namespace']['namespace'],
                            object_name=actual_obj['object']['name'],
                            client=client)

        ns_objects = self.create_objects()

        # Get object - member role from 'project' can access all
        # objects of it's own & only objects having public namespace of
        # 'alt_project'
        for obj in ns_objects:
            assertObjectGet(obj, self.project_id, self.objects_client)

    def test_list_objects(self):
        ns_objects = self.create_objects()

        # list objects - member role from 'project' can access all
        # objects of it's own & only objects having public namespace of
        # 'alt_project'
        for obj in ns_objects:
            self.assertObjectsList(obj, self.objects_client, self.project_id)

    def test_update_object(self):

        def assertObjectUpdate(actual_object, owner, client):
            ns = actual_object['namespace']
            expected_status = exceptions.Forbidden
            if (ns['visibility'] == 'private' and
                    ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('update_namespace_object',
                            expected_status=expected_status,
                            name=actual_object['object']['name'],
                            description=data_utils.arbitrary_string(),
                            namespace=actual_object['namespace']['namespace'],
                            object_name=actual_object['object']['name'],
                            client=client)

        ns_objects = self.create_objects()
        # Make sure non admin role of 'project' not allowed to
        # update objects
        for obj in ns_objects:
            assertObjectUpdate(obj, self.project_id, self.objects_client)

    def test_delete_object(self):

        def assertObjectDelete(actual_obj, owner, client):
            ns = actual_obj['namespace']
            expected_status = exceptions.Forbidden
            if (ns['visibility'] == 'private' and
                    ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('delete_namespace_object',
                            expected_status=expected_status,
                            namespace=actual_obj['namespace']['namespace'],
                            object_name=actual_obj['object']['name'],
                            client=client)

        ns_objects = self.create_objects()
        # Make sure non admin role of 'project' not allowed to
        # delete objects
        for obj in ns_objects:
            assertObjectDelete(obj, self.project_id, self.objects_client)


class PropertiesProjectMemberTests(rbac_base.MetadefV2RbacPropertiesTest,
                                   rbac_base.MetadefV2RbacPropertiesTemplate):

    credentials = ['project_member', 'project_alt_member',
                   'project_admin', 'project_alt_admin', 'primary']

    def test_create_property(self):
        namespaces = self.create_namespaces()

        def assertPropertyCreate(namespace, client, owner=None):
            property_name = "prop_of_%s" % (namespace['namespace'])
            expected_status = exceptions.Forbidden
            if not (namespace['visibility'] == 'public' or
                    namespace['owner'] == owner):
                expected_status = exceptions.NotFound

            self.do_request('create_namespace_property',
                            expected_status=expected_status,
                            namespace=namespace['namespace'],
                            title="property",
                            type='integer',
                            name=property_name,
                            client=client)
        # Make sure non admin role of 'project' forbidden to
        # create properties
        for namespace in namespaces:
            assertPropertyCreate(namespace, self.properties_client,
                                 self.project_id)

    def test_get_properties(self):
        ns_properties = self.create_properties()

        def assertPropertyGet(actual_prop, client, owner=None):
            ns = actual_prop['namespace']
            expected_status = 200
            if (ns['visibility'] != 'public' and
                    ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('show_namespace_properties',
                            expected_status=expected_status,
                            namespace=ns['namespace'],
                            property_name=actual_prop['property']['name'],
                            client=client)

        # Get property - member role from 'project' can access all
        # properties of it's own & only propertys having public namespace of
        # 'alt_project'
        for prop in ns_properties:
            assertPropertyGet(prop, self.properties_client, self.project_id)

    def test_list_properties(self):
        ns_properties = self.create_properties()

        # list properties - member role from 'project' can access all
        # properties of it's own & only propertys having public namespace of
        # 'alt_project'
        for prop in ns_properties:
            self.assertPropertyList(prop, self.properties_client,
                                    self.project_id)

    def test_update_properties(self):
        ns_properties = self.create_properties()

        def assertPropertyUpdate(actual_prop, client, owner=None):
            ns = actual_prop['namespace']
            expected_status = exceptions.Forbidden
            if (ns['visibility'] != 'public' and
                    ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('update_namespace_properties',
                            expected_status=expected_status,
                            name=actual_prop['property']['name'],
                            description=data_utils.arbitrary_string(),
                            namespace=ns['namespace'],
                            property_name=actual_prop['property']['name'],
                            title="UPDATE_Property",
                            type="string",
                            client=client)

        # Make sure non admin role of 'project' not allowed to
        # update properties
        for prop in ns_properties:
            assertPropertyUpdate(prop, self.properties_client,
                                 self.project_id)

    def test_delete_properties(self):
        ns_properties = self.create_properties()

        def assertPropertyDelete(actual_prop, client, owner=None):
            ns = actual_prop['namespace']
            expected_status = exceptions.Forbidden
            if (ns['visibility'] != 'public' and
                    ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('delete_namespace_property',
                            expected_status=expected_status,
                            namespace=ns['namespace'],
                            property_name=actual_prop['property']['name'],
                            client=client)

        # Make sure non admin role of 'project' not allowed to
        # delete properties
        for prop in ns_properties:
            assertPropertyDelete(prop, self.properties_client,
                                 self.project_id)


class TagsProjectMemberTests(rbac_base.MetadefV2RbacTagsTest,
                             rbac_base.MetadefV2RbacTagsTemplate):

    credentials = ['project_member', 'project_alt_member',
                   'project_admin', 'project_alt_admin', 'primary']

    def test_create_tag(self):
        namespaces = self.create_namespaces()

        def assertTagsCreate(namespace, client, owner, multiple_tags=False):
            expected_status = exceptions.Forbidden
            if (namespace['visibility'] != 'public' and
                    namespace['owner'] != owner):
                expected_status = exceptions.NotFound

            if multiple_tags:
                multiple_tags = [{"name": "tag1"}, {"name": "tag2"},
                                 {"name": "tag3"}]
                self.do_request('create_namespace_tags',
                                expected_status=expected_status,
                                namespace=namespace['namespace'],
                                tags=multiple_tags,
                                client=client)
            else:
                tag_name = "tag_of_%s" % (namespace['namespace'])
                self.do_request('create_namespace_tag',
                                expected_status=expected_status,
                                namespace=namespace['namespace'],
                                tag_name=tag_name,
                                client=client)

        # Make sure non admin role of 'project' forbidden to
        # create tags
        for namespace in namespaces:
            assertTagsCreate(namespace, self.tags_client, self.project_id)

            # Create Multiple Tags
            assertTagsCreate(namespace, self.tags_client, self.project_id,
                             multiple_tags=True)

    def test_get_tags(self):
        namespaces = self.create_namespaces()
        ns_tags = self.create_tags(namespaces)

        def assertTagGet(actual_tag, client, owner=None):
            ns = actual_tag['namespace']
            expected_status = 200
            if (ns['visibility'] != 'public' and ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('show_namespace_tag',
                            expected_status=expected_status,
                            namespace=ns['namespace'],
                            tag_name=actual_tag['tag']['name'],
                            client=client)

        # Get tag - member role from 'project' can access all
        # tags of it's own & only tags having public namespace of
        # 'alt_project'
        for tag in ns_tags:
            assertTagGet(tag, self.tags_client, self.project_id)

    def test_list_tags(self):
        namespaces = self.create_namespaces()
        ns_tags = self.create_tags(namespaces)

        # list tags - member role from 'project' can access all
        # tags of it's own & only tags having public namespace of
        # 'alt_project'
        for tag in ns_tags:
            self.assertTagsList(tag, self.tags_client, self.project_id)

    def test_update_tags(self):
        namespaces = self.create_namespaces()
        ns_tags = self.create_tags(namespaces)

        def assertTagUpdate(tag, client, owner):
            ns = tag['namespace']
            expected_status = exceptions.Forbidden
            if (ns['visibility'] != 'public' and ns['owner'] != owner):
                expected_status = exceptions.NotFound

            self.do_request('update_namespace_tag',
                            expected_status=expected_status,
                            name=data_utils.arbitrary_string(),
                            namespace=tag['namespace']['namespace'],
                            tag_name=tag['tag']['name'],
                            client=client)

        # Make sure non admin role of 'project' not allowed to
        # update tags
        for tag in ns_tags:
            assertTagUpdate(tag, self.tags_client, self.project_id)

    def test_delete_tags(self):
        namespaces = self.create_namespaces()
        ns_tags = self.create_tags(namespaces)

        def assertTagsDelete(actual_tag, client, owner, multiple_tags=False):
            ns = tag['namespace']
            expected_status = exceptions.Forbidden
            if (ns['visibility'] != 'public' and ns['owner'] != owner):
                expected_status = exceptions.NotFound

            if multiple_tags:
                self.do_request('delete_namespace_tags',
                                expected_status=expected_status,
                                namespace=ns['namespace'],
                                client=client)
            else:
                self.do_request('delete_namespace_tag',
                                expected_status=expected_status,
                                namespace=ns['namespace'],
                                tag_name=actual_tag['tag']['name'],
                                client=client)

        # Make sure non admin role of 'project' not allowed to
        # delete tags
        for tag in ns_tags:
            assertTagsDelete(tag, self.tags_client, self.project_id)

        # Create Multiple Tags
        ns_multiple_tags = self.create_tags(namespaces, multiple_tags=True)
        # Make sure non admin role of 'project' not allowed to
        # delete multiple tags
        for tags in ns_multiple_tags:
            assertTagsDelete(tags, self.tags_client, self.project_id,
                             multiple_tags=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from glance_tempest_plugin.tests.rbac.v2 import base as rbac_base

CONF = config.CONF


class ImageProjectAdminTests(rbac_base.ImageV2RbacImageTest,
                             rbac_base.ImageV2RbacTemplate):

    credentials = ['project_admin', 'system_admin', 'project_alt_admin']

    @decorators.idempotent_id('025eea27-fa86-44a9-85a2-91295842f808')
    def test_create_image(self):
        image = self.do_request('create_image', expected_status=201,
                                **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        image = self.do_request('create_image', expected_status=201,
                                **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        image = self.do_request('create_image', expected_status=201,
                                **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        image = self.do_request('create_image', expected_status=201,
                                **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to create image.
            self.do_request('create_image',
                            expected_status=exceptions.Forbidden,
                            client=self.os_system_admin.image_client_v2,
                            **self.image(visibility='public'))

    @decorators.idempotent_id('61fd8b5e-8a0b-46ca-91c4-6c2c2d35039d')
    def test_get_image(self):
        # Ensure users can get private images owned by their project.
        project_id = self.persona.credentials.project_id
        project_member = self.setup_user_client(project_id=project_id)
        image = project_member.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])

        image = project_member.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to get image.
            self.do_request('show_image', expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to get image.
            self.do_request('show_image', expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])
        project_id = self.persona.credentials.project_id
        self.admin_client.image_member_client_v2.create_image_member(
            image['id'], member=project_id)
        self.addCleanup(
            self.admin_client.image_member_client_v2.delete_image_member,
            image['id'], project_id)

        self.do_request('show_image', image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image', image_id=image['id'])

        self.do_request('show_image', expected_status=exceptions.NotFound,
                        image_id=data_utils.rand_uuid())

    @decorators.idempotent_id('d0c18f80-6168-4d98-a86e-c09d28d83bb0')
    def test_list_images(self):
        project_id = self.persona.credentials.project_id
        project_member = self.setup_user_client(project_id=project_id)
        project_client = self.setup_user_client()

        # Create a private image in the project
        private_image_in_project = project_member.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        private_image_in_project['id'])

        # Create a private image without an owner
        private_image_no_owner = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        private_image_no_owner['id'])

        # Create a private image in another project
        private_image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        private_image['id'])

        # Create a public image
        public_image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image,
                        public_image['id'])

        resp = self.do_request('list_images', expected_status=200)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to list images.
            self.do_request('list_images',
                            expected_status=exceptions.Forbidden,
                            client=self.os_system_admin.image_client_v2)
        image_ids = set(image['id'] for image in resp['images'])

        self.assertIn(private_image_no_owner['id'], image_ids)
        self.assertIn(private_image['id'], image_ids)
        self.assertIn(public_image['id'], image_ids)
        self.assertIn(private_image_in_project['id'], image_ids)

        # Create a shared image in another project
        shared_image_1 = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        shared_image_2 = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        shared_image_1['id'])
        self.addCleanup(self.admin_images_client.delete_image,
                        shared_image_2['id'])

        # Share the image from the other project with the user's project
        project_id = self.persona.credentials.project_id
        self.admin_client.image_member_client_v2.create_image_member(
            shared_image_1['id'], member=project_id)
        self.addCleanup(
            self.admin_client.image_member_client_v2.delete_image_member,
            shared_image_1['id'], project_id)

        # List images and assert the shared image is not in the list of images
        # because it hasn't been accepted, yet.
        resp = self.do_request('list_images', expected_status=200)
        image_ids = set(image['id'] for image in resp['images'])
        self.assertIn(shared_image_1['id'], image_ids)
        self.assertIn(shared_image_2['id'], image_ids)

        # Accept the image and ensure it's returned in the list of images
        project_member.image_member_client_v2.update_image_member(
            shared_image_1['id'], project_id, status='accepted')
        resp = self.do_request('list_images', expected_status=200)
        image_ids = set(image['id'] for image in resp['images'])
        self.assertIn(shared_image_1['id'], image_ids)
        self.assertIn(shared_image_2['id'], image_ids)

    @decorators.idempotent_id('9e9f7fd6-e93c-402c-9f3c-177fede8f645')
    def test_update_image(self):
        image = self.client.create_image(**self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)

        image = self.client.create_image(**self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to update image.
            self.do_request('update_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'], patch=patch_body)
        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)

        image = self.client.create_image(**self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to update image.
            self.do_request('update_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'], patch=patch_body)
        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to update image.
            self.do_request('update_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'], patch=patch_body)
        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=200,
                        image_id=image['id'], patch=patch_body)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to update image.
            self.do_request('update_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'], patch=patch_body)

    @decorators.idempotent_id('947f1ae1-c5b6-4552-89e3-1078ca722be4')
    def test_upload_image(self):
        file_contents = data_utils.random_bytes()
        image_data = io.BytesIO(file_contents)

        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        # upload file for private image - pass
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file', expected_status=204,
                        image_id=image['id'], data=image_data)

        # upload file for shared image - pass
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file', expected_status=204,
                        image_id=image['id'], data=image_data)

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file', image_id=image['id'],
                        expected_status=204, data=image_data)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to store image file.
            self.do_request('store_image_file',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'], data=image_data)

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file', expected_status=204,
                        image_id=image['id'], data=image_data)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to store image file.
            self.do_request('store_image_file',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'], data=image_data)
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file', expected_status=204,
                        image_id=image['id'], data=image_data)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to store image file.
            self.do_request('store_image_file',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'], data=image_data)
        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file', expected_status=204,
                        image_id=image['id'], data=image_data)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to store image file.
            self.do_request('store_image_file',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'], data=image_data)
        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file', expected_status=204,
                        image_id=image['id'], data=image_data)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to store image file.
            self.do_request('store_image_file',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'], data=image_data)

    @decorators.idempotent_id('24891c04-28ca-41f9-92d1-c06d8ba4b83d')
    def test_download_image(self):
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])

        project_client = self.setup_user_client()

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to show image file.
            self.do_request('show_image_file',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to show image file.
            self.do_request('show_image_file',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image_file', expected_status=204,
                        image_id=image['id'])

    @decorators.idempotent_id('e45899d6-7e16-4cbf-b800-31c05e6caf8c')
    def test_delete_image(self):

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to delete image.
            self.do_request('delete_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to delete image.
            self.do_request('delete_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.client.create_image(
            **self.image(visibility='private'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to delete image.
            self.do_request('delete_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.client.create_image(
            **self.image(visibility='shared'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='community'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to delete image.
            self.do_request('delete_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.client.create_image(
            **self.image(visibility='community'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.do_request('delete_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to delete image.
            self.do_request('delete_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

    @decorators.idempotent_id('ec3da4dc-f478-4a70-8799-db0814e340f4')
    def test_add_image_member(self):
        # Create a user with the member role in a separate project.
        other_project_member_client = self.setup_user_client()
        other_project_id = other_project_member_client.credentials.project_id

        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        # Make sure the persona user can add image members to images they
        # create.
        self.do_request('create_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member=other_project_id)

    @decorators.idempotent_id('8719285b-5b7b-48b8-ba5e-2bc7e3535025')
    def test_get_image_member(self):
        # Create a user with the member role in a separate project.
        project_one_client = self.setup_user_client()
        project_one_id = project_one_client.credentials.project_id

        # Create an image and share it with the other project.
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=project_one_id)
        self.do_request('show_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member_id=project_one_id)

        # Create an image associated to a separate project and make project_one
        # a member.
        project_two_client = self.setup_user_client()
        image = project_two_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_two_client.image_member_client_v2.create_image_member(
            image['id'], member=project_one_id)
        self.do_request('show_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member_id=project_one_id)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to show image member.
            self.do_request('show_image_member',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_member_client_v2,
                            image_id=image['id'], member_id=project_one_id)

    @decorators.idempotent_id('daaef0c5-1172-457b-b1a3-0736b64c8426')
    def test_list_image_members(self):
        # Stash the project_id for this persona
        project_id = self.persona.credentials.project_id

        other_member_client = self.setup_user_client()
        other_member_project_id = other_member_client.credentials.project_id

        # Create an image as another user in a separate project and share that
        # image with this user and the other member.
        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        for p_id in [project_id, other_member_project_id]:
            project_client.image_member_client_v2.create_image_member(
                image['id'], member=p_id)
        resp = self.do_request('list_image_members',
                               client=self.persona.image_member_client_v2,
                               expected_status=200, image_id=image['id'])
        members = set(m['member_id'] for m in resp['members'])
        self.assertIn(project_id, members)
        self.assertIn(other_member_project_id, members)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to list image members.
            self.do_request('list_image_members',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_member_client_v2,
                            image_id=image['id'])

    @decorators.idempotent_id('2baaaca0-6335-4219-9bd9-207a5cfda6a2')
    def test_update_image_member(self):
        project_client = self.setup_user_client()
        # Create a shared image in a separate project and share the with the
        # persona project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=self.persona.credentials.project_id)

        # Make sure the persona user can accept the image.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member_id=self.persona.credentials.project_id,
                        status='accepted')

        # Make sure the persona user can reject the image.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member_id=self.persona.credentials.project_id,
                        status='rejected')

        # Create another shared image in a separate project (not the persona
        # user's project).
        member_client = self.setup_user_client()
        member_project_id = member_client.credentials.project_id
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])

        # Share the image with another project.
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)

        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member_id=member_project_id, status='accepted')

        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=200, image_id=image['id'],
                        member_id=member_project_id, status='rejected')
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to update image member.
            self.do_request('update_image_member',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_member_client_v2,
                            image_id=image['id'], member_id=member_project_id,
                            status='rejected')

    @decorators.idempotent_id('7f0a8e2b-b655-416a-914b-9615cff18bbf')
    def test_delete_image_member(self):
        # Create a user with authorization on another project.
        member_client = self.setup_user_client()
        member_project_id = member_client.credentials.project_id

        # Create a separate user with authorization on the persona project.
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        # Create an image in the persona project and share it with the member.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)

        # Make sure we, as the image owners, can remove membership to that
        # image.
        self.do_request('delete_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=204,
                        image_id=image['id'], member_id=member_project_id)

        # Create a new user with authorization on a separate project.
        project_client = self.setup_user_client()

        # Create an image in that project and share it with the member project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)

        self.do_request('delete_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=204, image_id=image['id'],
                        member_id=member_project_id)
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to delete image member.
            self.do_request('delete_image_member',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_member_client_v2,
                            image_id=image['id'], member_id=member_project_id)

    @decorators.idempotent_id('dfc73f6f-bf91-4b6a-8482-acc8c436e066')
    def test_deactivate_image(self):
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        file_contents = data_utils.random_bytes()
        image_data = io.BytesIO(file_contents)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        self.do_request('deactivate_image', expected_status=204,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        self.do_request('deactivate_image', expected_status=204,
                        image_id=image['id'])

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        self.do_request('deactivate_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to deactivate image.
            self.do_request('deactivate_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        self.do_request('deactivate_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to deactivate image.
            self.do_request('deactivate_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.do_request('deactivate_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to deactivate image.
            self.do_request('deactivate_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.do_request('deactivate_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to deactivate image.
            self.do_request('deactivate_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

    @decorators.idempotent_id('3cbef53c-ab8a-4343-b993-8f9e14ab90d1')
    def test_reactivate_image(self):
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        file_contents = data_utils.random_bytes()
        image_data = io.BytesIO(file_contents)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])
        self.do_request('reactivate_image', expected_status=204,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])
        self.do_request('reactivate_image', expected_status=204,
                        image_id=image['id'])

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])
        self.do_request('reactivate_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to reactivate image.
            self.do_request('reactivate_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])
        self.do_request('reactivate_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to reactivate image.
            self.do_request('reactivate_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.admin_images_client.deactivate_image(image['id'])
        self.do_request('reactivate_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to reactivate image.
            self.do_request('reactivate_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.admin_images_client.deactivate_image(image['id'])
        self.do_request('reactivate_image', expected_status=204,
                        image_id=image['id'])
        if not CONF.glance_api.system_scope_supported:
            # Check that system user is not permitted to reactivate image.
            self.do_request('reactivate_image',
                            expected_status=exceptions.NotFound,
                            client=self.os_system_admin.image_client_v2,
                            image_id=image['id'])


class NamespacesProjectAdminTests(rbac_base.MetadefV2RbacNamespaceTest,
                                  rbac_base.MetadefV2RbacNamespaceTemplate):

    credentials = ['project_admin', 'project_alt_admin', 'primary']

    def test_list_namespaces(self):
        actual_namespaces = self.create_namespaces()

        # Get above created namespace by admin role
        resp = self.do_request('list_namespaces', expected_status=200,
                               client=self.admin_namespace_client)

        self.assertListNamespaces(actual_namespaces, resp)

    def test_get_namespace(self):
        actual_namespaces = self.create_namespaces()

        # Get above created namespace by admin role
        for ns in actual_namespaces:
            resp = self.do_request('show_namespace', expected_status=200,
                                   namespace=ns['namespace'],
                                   client=self.admin_namespace_client)
            self.assertEqual(ns['namespace'], resp['namespace'])

    def test_create_namespace(self):
        # As this is been covered in other tests for admin role,
        # skipping to test only create namespaces seperately.
        pass

    def test_update_namespace(self):
        actual_namespaces = self.create_namespaces()

        # Updating the above created namespace by admin role
        for ns in actual_namespaces:
            resp = self.do_request(
                'update_namespace', expected_status=200,
                namespace=ns['namespace'],
                client=self.admin_namespace_client,
                description=data_utils.arbitrary_string(base_text="updated"))
            self.assertNotEqual(ns['description'], resp['description'])

    def test_delete_namespace(self):
        actual_namespaces = self.create_namespaces()

        # Deleting the above created namespace by admin role
        for ns in actual_namespaces:
            self.do_request('delete_namespace', expected_status=204,
                            namespace=ns['namespace'],
                            client=self.admin_namespace_client,)

            # Verify the namespaces are deleted successfully
            self.do_request('show_namespace',
                            expected_status=exceptions.NotFound,
                            namespace=ns['namespace'],
                            client=self.admin_namespace_client,)


class RSTypesProjectAdminTests(rbac_base.MetadefV2RbacResourceTypeTest,
                               rbac_base.MetadefV2RbacResourceTypeTemplate):

    credentials = ['project_admin', 'project_alt_admin', 'primary']

    def test_create_resource_type(self):
        # As this is been covered in other tests for admin role,
        # skipping to test only create resource types separately.
        pass

    def test_get_resource_type(self):
        ns_rs_types = self.create_resource_types()

        # Get all metadef resource types with admin role of 'project'
        for rs_type in ns_rs_types:
            resp = self.do_request(
                'list_resource_type_association',
                expected_status=200,
                client=self.resource_types_client,
                namespace_id=rs_type['namespace']['namespace'])
            self.assertEqual(rs_type['resource_type']['name'],
                             resp['resource_type_associations'][0]['name'])

    def test_list_resource_types(self):
        ns_rs_types = self.create_resource_types()

        # list resource types - with admin role of 'project'
        resp = self.do_request('list_resource_types',
                               expected_status=200,
                               client=self.resource_types_client)

        # Verify that admin role of 'project' will be able to view available
        # resource types
        self.assertRSTypeList(ns_rs_types, resp)

    def test_delete_resource_type(self):
        ns_rs_types = self.create_resource_types()

        # delete all metadef resource types with admin role of 'project'
        for rs_type in ns_rs_types:
            self.do_request('delete_resource_type_association',
                            expected_status=204,
                            namespace_id=rs_type['namespace']['namespace'],
                            resource_name=rs_type['resource_type']['name'],
                            client=self.resource_types_client)

            # Verify the resource types is deleted successfully
            resp = self.do_request(
                'list_resource_type_association', expected_status=200,
                client=self.resource_types_client,
                namespace_id=rs_type['namespace']['namespace'])
            self.assertEqual([], resp['resource_type_associations'])


class ObjectsProjectAdminTests(rbac_base.MetadefV2RbacObjectsTest,
                               rbac_base.MetadefV2RbacObjectsTemplate):

    credentials = ['project_admin', 'project_alt_admin', 'primary']

    def test_get_object(self):
        ns_objects = self.create_objects()

        # Get all metadef objects with admin role
        for obj in ns_objects:
            resp = self.do_request(
                'show_namespace_object',
                expected_status=200,
                client=self.objects_client,
                namespace=obj['namespace']['namespace'],
                object_name=obj['object']['name'])
            self.assertEqual(obj['object']['name'], resp['name'])

    def test_list_objects(self):
        ns_objects = self.create_objects()

        # list all metadef objects with admin role
        for obj in ns_objects:
            self.assertObjectsList(obj, self.objects_client)

    def test_update_object(self):
        ns_objects = self.create_objects()

        # update all metadef objects with admin role of 'project'
        for obj in ns_objects:
            resp = self.do_request(
                'update_namespace_object',
                expected_status=200,
                namespace=obj['namespace']['namespace'],
                client=self.objects_client,
                object_name=obj['object']['name'],
                name=obj['object']['name'],
                description=data_utils.arbitrary_string(base_text="updated"))
            self.assertNotEqual(obj['object']['description'],
                                resp['description'])

    def test_delete_object(self):
        ns_objects = self.create_objects()
        # delete all metadef objects with admin role of 'project'
        for obj in ns_objects:
            self.do_request('delete_namespace_object',
                            expected_status=204,
                            namespace=obj['namespace']['namespace'],
                            object_name=obj['object']['name'],
                            client=self.objects_client)

            # Verify the object is deleted successfully
            self.do_request('show_namespace_object',
                            expected_status=exceptions.NotFound,
                            client=self.objects_client,
                            namespace=obj['namespace']['namespace'],
                            object_name=obj['object']['name'])

    def test_create_object(self):
        # As this is been covered in other tests for admin role,
        # skipping to test only create objects seperately.
        pass


class PropertiesProjectAdminTests(rbac_base.MetadefV2RbacPropertiesTest,
                                  rbac_base.MetadefV2RbacPropertiesTemplate):

    credentials = ['project_admin', 'project_alt_admin', 'primary']

    def test_create_property(self):
        # As this is been covered in other tests for admin role,
        # skipping to test only create properties separately.
        pass

    def test_get_properties(self):
        ns_properties = self.create_properties()

        # Get all metadef properties with admin role of 'project'
        for prop in ns_properties:
            resp = self.do_request(
                'show_namespace_properties',
                expected_status=200,
                client=self.properties_client,
                namespace=prop['namespace']['namespace'],
                property_name=prop['property']['name'])
            self.assertEqual(prop['property'], resp)

    def test_list_properties(self):
        ns_properties = self.create_properties()
        # list all metadef properties with admin role of 'project'
        for prop in ns_properties:
            self.assertPropertyList(prop, self.properties_client)

    def test_update_properties(self):
        ns_properties = self.create_properties()

        # update all metadef properties with admin role of 'project'
        for prop in ns_properties:
            resp = self.do_request(
                'update_namespace_properties',
                expected_status=200,
                namespace=prop['namespace']['namespace'],
                client=self.properties_client,
                title="UPDATE_Property",
                property_name=prop['property']['name'],
                name=prop['property']['name'],
                type="string")
            self.assertNotEqual(prop['property']['title'],
                                resp['title'])

    def test_delete_properties(self):
        ns_properties = self.create_properties()

        # delete all metadef properties with admin role of 'project'
        for prop in ns_properties:
            self.do_request('delete_namespace_property',
                            expected_status=204,
                            namespace=prop['namespace']['namespace'],
                            property_name=prop['property']['name'],
                            client=self.properties_client)

            # Verify the property is deleted successfully
            self.do_request('show_namespace_properties',
                            expected_status=exceptions.NotFound,
                            client=self.properties_client,
                            namespace=prop['namespace']['namespace'],
                            property_name=prop['property']['name'])


class TagsProjectAdminTests(rbac_base.MetadefV2RbacTagsTest,
                            rbac_base.MetadefV2RbacTagsTemplate):

    credentials = ['project_admin', 'project_alt_admin', 'primary']

    def test_create_tag(self):
        # As this is been covered in other tests for admin role,
        # skipping to test only create properties separately.
        pass

    def test_get_tags(self):
        namespaces = self.create_namespaces()
        ns_tags = self.create_tags(namespaces)

        # Get all metadef tags with admin role of 'project'
        for tag in ns_tags:
            resp = self.do_request(
                'show_namespace_tag',
                expected_status=200,
                client=self.tags_client,
                namespace=tag['namespace']['namespace'],
                tag_name=tag['tag']['name'])
            self.assertEqual(tag['tag']['name'], resp['name'])

    def test_list_tags(self):
        namespaces = self.create_namespaces()
        ns_tags = self.create_tags(namespaces)
        # list all metadef tags with admin role of 'project'
        for tag in ns_tags:
            self.assertTagsList(tag, self.tags_client)

    def test_update_tags(self):
        namespaces = self.create_namespaces()
        ns_tags = self.create_tags(namespaces)

        # update all metadef tags with admin role of 'project'
        for tag in ns_tags:
            resp = self.do_request(
                'update_namespace_tag',
                expected_status=200,
                namespace=tag['namespace']['namespace'],
                client=self.tags_client,
                tag_name=tag['tag']['name'],
                name=data_utils.arbitrary_string(base_text="updated-name"))
            self.assertNotEqual(tag['tag']['name'], resp['name'])

    def test_delete_tags(self):
        namespaces = self.create_namespaces()
        ns_tags = self.create_tags(namespaces)

        def assertDeleteTags(tag, client, multiple_tags=False):
            namespace = tag['namespace']['namespace']
            if multiple_tags:
                self.do_request('delete_namespace_tags',
                                expected_status=204,
                                namespace=namespace,
                                client=client)
                # Verify the tags are deleted successfully
                resp = self.do_request('list_namespace_tags',
                                       client=client,
                                       namespace=namespace)
                self.assertEqual(0, len(resp['tags']))
            else:
                tag_name = tag['tag']['name']
                self.do_request('delete_namespace_tag',
                                expected_status=204,
                                namespace=namespace,
                                tag_name=tag_name,
                                client=client)

                # Verify the tag is deleted successfully
                self.do_request('show_namespace_tag',
                                expected_status=exceptions.NotFound,
                                client=client,
                                namespace=namespace,
                                tag_name=tag_name)

        # delete all metadef tags with admin role of 'project'
        for tag in ns_tags:
            assertDeleteTags(tag, self.tags_client)

        # Create multiple tags
        ns_multiple_tags = self.create_tags(namespaces, multiple_tags=True)
        # delete all metadef multiple tags with admin role of 'project'
        for tags in ns_multiple_tags:
            assertDeleteTags(tags, self.tags_client, multiple_tags=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from glance_tempest_plugin.tests.rbac.v2 import test_project_members_tests

CONF = config.CONF


class ImagesProjectReaderTests(
        test_project_members_tests.ImagesProjectMemberTests):

    credentials = ['project_reader', 'project_admin', 'system_admin',
                   'admin', 'project_alt_admin', 'project_alt_member',
                   'project_alt_reader']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.alt_project_images_client = (
            cls.os_project_alt_reader.image_client_v2)

    @decorators.idempotent_id('5e151433-5901-45ec-9451-ed3170c299cb')
    def test_create_image(self):
        # Project readers can't create images.
        self.do_request('create_image', expected_status=exceptions.Forbidden,
                        **self.image(visibility='private'))

        self.do_request('create_image', expected_status=exceptions.Forbidden,
                        **self.image(visibility='shared'))

        self.do_request('create_image', expected_status=exceptions.Forbidden,
                        **self.image(visibility='community'))

        self.do_request('create_image', expected_status=exceptions.Forbidden,
                        **self.image(visibility='public'))

    @decorators.idempotent_id('6de1e04c-d0cd-45b3-8007-3712bbca817d')
    def test_list_images(self):
        super().test_list_images()

    @decorators.idempotent_id('f402e6a2-7cc9-46c0-b6c2-a235f5512788')
    def test_get_image(self):
        super().test_get_image()

    @decorators.idempotent_id('f2be46ee-317b-4825-9d3d-bd23a1fb7858')
    def test_update_image(self):
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'], patch=patch_body)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'], patch=patch_body)

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        # This fails because getting an image outside the user's project
        # returns a 404.
        self.do_request('update_image', expected_status=exceptions.NotFound,
                        image_id=image['id'], patch=patch_body,
                        client=self.alt_project_images_client)

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        # FIXME(lbragstad): This is different from the status code for
        # project-admininstrators because the database layer allows users with
        # the admin role the ability to view images outside their project, and
        # returns a 403 because it checks tenancy later. IMO, this return code
        # shouldn't be any different from project users.
        self.do_request('update_image', expected_status=exceptions.NotFound,
                        image_id=image['id'], patch=patch_body,
                        client=self.alt_project_images_client)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        # Same comment as above with updating private images for other
        # projects.
        self.do_request('update_image', expected_status=exceptions.NotFound,
                        image_id=image['id'], patch=patch_body)

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'], patch=patch_body)

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        name = data_utils.rand_name('new-image-name')
        patch_body = [dict(replace='/name', value=name)]
        self.do_request('update_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'], patch=patch_body)

    @decorators.idempotent_id('b7ac2883-f569-4032-a35b-a79ef1277582')
    def test_upload_image(self):
        file_contents = data_utils.random_bytes()
        image_data = io.BytesIO(file_contents)

        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'], data=image_data)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'], data=image_data)

        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], data=image_data,
                        client=self.alt_project_images_client)

        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], data=image_data)

        # This fails because the user can't actually see the image since it's
        # not shared with them.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], data=image_data)

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'], data=image_data)

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('store_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'], data=image_data)

    @decorators.idempotent_id('9067339d-c64b-4e4d-bc0e-a52cd1365ea3')
    def test_download_image(self):
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        project_client = self.setup_user_client()

        # Fail to download an image for another project because we can't find
        # it.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'])

        # Fail to download an image for another project because we can't find
        # it.
        image = self.admin_images_client.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image,
                        image['id'])
        self.do_request('show_image_file',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'],
                        client=self.alt_project_images_client)

        # Fail to download an image for another project because we can't find
        # it.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image_file',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('show_image_file',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

    @decorators.idempotent_id('e8c3382c-c547-49c1-a92e-fa1d6378d4df')
    def test_delete_image(self):
        # Project readers can't delete images outside their project. This is
        # returned as an HTTP 404 instead of an HTTP 403 because they can't
        # find the image they're trying to delete.
        project_client = self.setup_user_client()
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('delete_image', expected_status=exceptions.NotFound,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('delete_image', expected_status=exceptions.NotFound,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('delete_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        # Project readers can't delete images inside their project, regardless
        # of the image state (private, shared, community, or public).
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('delete_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('delete_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('delete_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        # Project readers can't delete public images.
        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('delete_image', expected_status=exceptions.Forbidden,
                        image_id=image['id'])

    @decorators.idempotent_id('8eaddc1a-f9d0-4a68-8fef-1951c328d01c')
    def test_add_image_member(self):
        # Create a new user with authorization on a separate project.
        member_client = self.setup_user_client()
        member_project_id = member_client.credentials.project_id

        # Create a new user with authorization on the persona user's project.
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        # Create a shared image and make sure we can't share the image with
        # other projects.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.do_request('create_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'],
                        member=member_project_id)

    @decorators.idempotent_id('e0feceab-dfc0-4d08-88be-81f5d225c72f')
    def test_update_image_member(self):
        # Create a new user with authorization on a separate project.
        project_client = self.setup_user_client()

        # Create a shared image in the other project and add the persona user's
        # project as a member.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=self.persona.credentials.project_id)

        # Make sure the user can't accept the image.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'],
                        member_id=self.persona.credentials.project_id,
                        status='accepted')

        # Make sure the user can't reject the change.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'],
                        member_id=self.persona.credentials.project_id,
                        status='rejected')

        # Create a new user with authorization on a separate project.
        member_client = self.setup_user_client()
        member_project_id = member_client.credentials.project_id

        # Have the original project client create a new shared image and share
        # it with the new member project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)

        # Make sure the user can't accept images for project they have no
        # authorization to know about.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], member_id=member_project_id,
                        status='accepted')

        # Make sure the user can't reject images for project they have no
        # authorization to know about.
        self.do_request('update_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], member_id=member_project_id,
                        status='rejected')

    @decorators.idempotent_id('864e275e-2238-4b0f-9039-9bc7aed22f92')
    def test_delete_image_member(self):
        # Create a new user with authorization on a separate project.
        member_client = self.setup_user_client()
        member_project_id = member_client.credentials.project_id

        # Create a new user with authorization on the persona user's project.
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)

        # Create a shared image in the persona user's project and share it with
        # the member project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)

        # Make sure the user can't delete image members.
        self.do_request('delete_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'], member_id=member_project_id)

        # Create a new user with authorization on a separate project.
        project_client = self.setup_user_client()

        # Creata a shared image in that new project and share it with the
        # member project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_member_client_v2.create_image_member(
            image['id'], member=member_project_id)

        # Make sure the user can't delete image members from images they have
        # no authorization to know about.
        self.do_request('delete_image_member',
                        client=self.persona.image_member_client_v2,
                        expected_status=exceptions.NotFound,
                        image_id=image['id'], member_id=member_project_id)

    @decorators.idempotent_id('310672b0-bf63-4ce0-b3c4-5230b8e7de31')
    def test_deactivate_image(self):
        # Create a new user with authorization on the persona user's project.
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        file_contents = data_utils.random_bytes()
        image_data = io.BytesIO(file_contents)

        # Create a new private image in the persona user's project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)

        # Make sure they can't deactivate images, even in their own project.
        self.do_request('deactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        # Create a new shared image in the persona user's project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)

        # Make sure they can't deactivate images, even in their own project.
        self.do_request('deactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        # Create a new user with authorization on a separate project.
        project_client = self.setup_user_client()

        # Create a private image in the new project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)

        # The user can't deactivate this image because they can't find it.
        self.do_request('deactivate_image',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'])

        # Create a shared image in the new project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)

        # The user can't deactivate this image because they can't find it.
        self.do_request('deactivate_image',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'])

        # Only administrators can deactivate community images.
        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.do_request('deactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        # Only administrators can deactivate public images.
        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.do_request('deactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

    @decorators.idempotent_id('0d1fc51c-d4c9-4dd5-9f21-c28b09d3f9ec')
    def test_reactivate_image(self):
        # Create a new user with authorization on the persona user's project.
        project_id = self.persona.credentials.project_id
        project_client = self.setup_user_client(project_id=project_id)
        file_contents = data_utils.random_bytes()
        image_data = io.BytesIO(file_contents)

        # Create a private image in the persona user's project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])

        # Make sure the user can't reactivate private images, even in their own
        # project.
        self.do_request('reactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        # Create a shared image in the persona user's project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])

        # Make sure the user can't reactivate shared images, even in their own
        # project.
        self.do_request('reactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        # Create a new user with authorization on a separate project.
        project_client = self.setup_user_client()

        # Create a private image in the new project.
        image = project_client.image_client_v2.create_image(
            **self.image(visibility='private'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])

        # The user can't reactivate this image because they can't find it.
        self.do_request('reactivate_image',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'])

        image = project_client.image_client_v2.create_image(
            **self.image(visibility='shared'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        project_client.image_client_v2.store_image_file(image['id'],
                                                        image_data)
        project_client.image_client_v2.deactivate_image(image['id'])

        # The user can't reactivate this image because they can't find it.
        self.do_request('reactivate_image',
                        expected_status=exceptions.NotFound,
                        image_id=image['id'])

        # Create a community image.
        image = self.admin_images_client.create_image(
            **self.image(visibility='community'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.admin_images_client.deactivate_image(image['id'])

        # Make sure the user can't reactivate community images.
        self.do_request('reactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])

        # Create a public image.
        image = self.admin_images_client.create_image(
            **self.image(visibility='public'))
        self.addCleanup(self.admin_images_client.delete_image, image['id'])
        self.admin_images_client.store_image_file(image['id'], image_data)
        self.admin_images_client.deactivate_image(image['id'])

        # Make sure the user can't reactivate public images.
        self.do_request('reactivate_image',
                        expected_status=exceptions.Forbidden,
                        image_id=image['id'])


class NamespacesProjectReaderTests(
        test_project_members_tests.NamespacesProjectMemberTests):
    credentials = ['project_reader', 'project_alt_reader',
                   'project_admin', 'project_alt_admin', 'primary']


class RSTypesProjectReaderTests(
        test_project_members_tests.RSTypesProjectMemberTests):

    credentials = ['project_reader', 'project_alt_reader', 'project_admin',
                   'project_alt_admin', 'primary']


class ObjectsProjectReaderTests(
        test_project_members_tests.ObjectsProjectMemberTests):

    credentials = ['project_reader', 'project_alt_reader', 'project_admin',
                   'project_alt_admin', 'primary']


class PropertiesProjectReaderTests(
        test_project_members_tests.PropertiesProjectMemberTests):

    credentials = ['project_reader', 'project_alt_reader',
                   'project_admin', 'project_alt_admin', 'primary']


class TagsProjectReaderTests(
        test_project_members_tests.TagsProjectMemberTests):

    credentials = ['project_reader', 'project_alt_reader',
                   'project_admin', 'project_alt_admin', 'primary']



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import abc

from tempest.api.image import base
from tempest import clients
from tempest import config
from tempest.lib import auth
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions

CONF = config.CONF


class RbacBaseTests(base.BaseV2ImageTest):

    identity_version = 'v3'

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not CONF.enforce_scope.glance:
            raise cls.skipException("enforce_scope is not enabled for "
                                    "glance, skipping RBAC tests")

    def do_request(self, method, expected_status=200, client=None, **payload):
        if not client:
            client = self.client
        if isinstance(expected_status, type(Exception)):
            self.assertRaises(expected_status,
                              getattr(client, method),
                              **payload)
        else:
            response = getattr(client, method)(**payload)
            self.assertEqual(response.response.status, expected_status)
            return response

    def setup_user_client(self, project_id=None):
        """Set up project user with its own client.

        This is useful for testing protection of resources in separate
        projects.

        Returns a client object and the user's ID.
        """
        user_dict = {
            'name': data_utils.rand_name('user'),
            'password': data_utils.rand_password(),
        }
        user_id = self.os_system_admin.users_v3_client.create_user(
            **user_dict)['user']['id']
        self.addCleanup(self.os_system_admin.users_v3_client.delete_user,
                        user_id)

        if not project_id:
            project_id = self.os_system_admin.projects_client.create_project(
                data_utils.rand_name())['project']['id']
            self.addCleanup(
                self.os_system_admin.projects_client.delete_project,
                project_id)

        member_role_id = self.os_system_admin.roles_v3_client.list_roles(
            name='member')['roles'][0]['id']
        self.os_system_admin.roles_v3_client.create_user_role_on_project(
            project_id, user_id, member_role_id)
        creds = auth.KeystoneV3Credentials(
            user_id=user_id,
            password=user_dict['password'],
            project_id=project_id)
        auth_provider = clients.get_auth_provider(creds)
        creds = auth_provider.fill_credentials()
        client = clients.Manager(credentials=creds)
        return client


def namespace(name, owner, visibility='private', protected=False):
    return {'namespace': name, 'visibility': visibility,
            'description': data_utils.arbitrary_string(),
            'display_name': name, 'owner': owner, 'protected': protected}


class RbacMetadefBase(RbacBaseTests):
    def create_namespaces(self):
        """Create private and public namespaces for different projects."""
        project_namespaces = []
        alt_namespaces = []
        for visibility in ['public', 'private']:
            project_ns = "%s_%s_%s" % (
                self.project_id,
                visibility,
                self.__class__.__name__)

            alt_ns = "%s_%s_%s" % (self.alt_project_id, visibility,
                                   self.__class__.__name__)

            project_namespace = \
                self.os_project_admin.namespaces_client.create_namespace(
                    **namespace(project_ns, self.project_id,
                                visibility=visibility))
            project_namespaces.append(project_namespace)
            self.addCleanup(
                test_utils.call_and_ignore_notfound_exc,
                self.os_project_admin.namespaces_client.delete_namespace,
                project_ns)

            alt_namespace = \
                self.os_project_admin.namespaces_client.create_namespace(
                    **namespace(alt_ns, self.alt_project_id,
                                visibility=visibility))
            alt_namespaces.append(alt_namespace)
            self.addCleanup(
                test_utils.call_and_ignore_notfound_exc,
                self.os_project_admin.namespaces_client.delete_namespace,
                alt_ns)

        return project_namespaces + alt_namespaces


class ImageV2RbacImageTest(RbacBaseTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.persona = getattr(cls, f'os_{cls.credentials[0]}')
        cls.client = cls.persona.image_client_v2
        # FIXME(lbragstad): This should use os_system_admin when glance
        # supports system scope.
        cls.admin_client = cls.os_project_admin
        cls.admin_images_client = cls.admin_client.image_client_v2

    @classmethod
    def setup_credentials(cls):
        super().setup_credentials()
        cls.os_primary = getattr(cls, f'os_{cls.credentials[0]}')

    def image(self, visibility=None):
        image = {}
        image['name'] = data_utils.rand_name('image')
        # NOTE(danms): It's technically possible that bare/raw is not allowed,
        # but this test suite cannot operate in that scenario since it does
        # not have valid image data to upload.
        if 'bare' not in CONF.image.container_formats:
            self.skipTest('This test requires the "bare" container format')
        if 'raw' not in CONF.image.disk_formats:
            self.skipTest('This test requires the "raw" container format')
        image['container_format'] = 'bare'
        image['disk_format'] = 'raw'
        image['visibility'] = visibility if visibility else 'private'
        image['ramdisk_uuid'] = '00000000-1111-2222-3333-444455556666'
        return image


class ImageV2RbacTemplate(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def test_create_image(self):
        """Test add_image policy.

        This test must check:
          * whether the persona can create a private image
          * whether the persona can create a shared image
          * whether the persona can create a community image
          * whether the persona can create a public image
        """

    @abc.abstractmethod
    def test_get_image(self):
        """Test get_image policy.

        This test must check:
          * whether a persona can get a private image
          * whether a persona can get a shared image
          * whether a persona can get a community image
          * whether a persona can get a public image
        """

    @abc.abstractmethod
    def test_list_images(self):
        """Test get_images policy.

        This test must check:
          * whether the persona can list private images within their project
          * whether the persona can list shared images
          * whether the persona can list community images
          * whether the persona can list public images
        """

    @abc.abstractmethod
    def test_update_image(self):
        """Test modify_image policy.

        This test must check:
          * whether the persona can modify private images
          * whether the persona can modify shared images
          * whether the persona can modify community images
          * whether the persona can modify public images
        """

    @abc.abstractmethod
    def test_upload_image(self):
        """Test upload_image policy.

        This test must check:
          * whether the persona can upload private images
          * whether the persona can upload shared images
          * whether the persona can upload community images
          * whether the persona can upload public images
        """

    @abc.abstractmethod
    def test_download_image(self):
        """Test download_image policy.

        This test must check:
          * whether the persona can download private images
          * whether the persona can download shared images
          * whether the persona can download community images
          * whether the persona can download public images
        """

    @abc.abstractmethod
    def test_delete_image(self):
        """Test delete_image policy.

        This test must check:
          * whether the persona can delete a private image
          * whether the persona can delete a shared image
          * whether the persona can delete a community image
          * whether the persona can delete a public image
          * whether the persona can delete an image outside their project
        """

    @abc.abstractmethod
    def test_add_image_member(self):
        pass

    @abc.abstractmethod
    def test_get_image_member(self):
        pass

    @abc.abstractmethod
    def test_list_image_members(self):
        pass

    @abc.abstractmethod
    def test_update_image_member(self):
        pass

    @abc.abstractmethod
    def test_delete_image_member(self):
        pass

    @abc.abstractmethod
    def test_deactivate_image(self):
        pass

    @abc.abstractmethod
    def test_reactivate_image(self):
        pass


class MetadefV2RbacNamespaceTest(RbacMetadefBase):
    @classmethod
    def setup_clients(cls):
        super(MetadefV2RbacNamespaceTest, cls).setup_clients()
        if 'project_member' in cls.credentials:
            persona = 'project_member'
            alt_persona = 'project_alt_member'
        elif 'project_reader' in cls.credentials:
            persona = 'project_reader'
            alt_persona = 'project_alt_reader'
        else:
            persona = 'project_admin'
            alt_persona = 'project_alt_admin'
        cls.persona = getattr(cls, 'os_%s' % persona)
        cls.alt_persona = getattr(cls, 'os_%s' % alt_persona)
        cls.project_id = cls.persona.namespaces_client.project_id
        cls.alt_project_id = cls.alt_persona.namespaces_client.project_id
        cls.admin_namespace_client = cls.os_project_admin.namespaces_client
        cls.namespace_client = cls.persona.namespaces_client
        cls.alt_namespace_client = cls.alt_persona.namespaces_client

    def assertListNamespaces(self, actual, expected, owner=None):
        expected_ns = set(ns['namespace'] for ns in expected['namespaces'])
        if owner:
            for ns in actual:
                if (ns['visibility'] == 'private' and
                        ns['owner'] != owner):
                    self.assertNotIn(ns['namespace'], expected_ns)
                else:
                    self.assertIn(ns['namespace'], expected_ns)
        else:
            for ns in actual:
                self.assertIn(ns['namespace'], expected_ns)


class MetadefV2RbacNamespaceTemplate(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def test_get_namespace(self):
        """Test get_metadef_namespace policy."""
        pass

    @abc.abstractmethod
    def test_list_namespaces(self):
        """Test get_metadef_namespaces policy."""
        pass

    @abc.abstractmethod
    def test_update_namespace(self):
        """Test modify_metadef_namespace policy."""
        pass

    @abc.abstractmethod
    def test_create_namespace(self):
        """Test add_metadef_namespace policy."""
        pass

    @abc.abstractmethod
    def test_delete_namespace(self):
        """Test delete_metadef_namespace policy."""
        pass


class MetadefV2RbacResourceTypeTest(RbacMetadefBase):
    @classmethod
    def setup_clients(cls):
        super(MetadefV2RbacResourceTypeTest, cls).setup_clients()
        if 'project_member' in cls.credentials:
            persona = 'project_member'
            alt_persona = 'project_alt_member'
        elif 'project_reader' in cls.credentials:
            persona = 'project_reader'
            alt_persona = 'project_alt_reader'
        else:
            persona = 'project_admin'
            alt_persona = 'project_alt_admin'
        cls.persona = getattr(cls, 'os_%s' % persona)
        cls.alt_persona = getattr(cls, 'os_%s' % alt_persona)
        cls.project_id = cls.persona.namespaces_client.project_id
        cls.alt_project_id = cls.alt_persona.namespaces_client.project_id
        cls.resource_types_client = cls.persona.resource_types_client

    def create_resource_types(self):
        # Create namespace for two different projects
        namespaces = self.create_namespaces()

        namespace_resource_types = []
        for ns in namespaces:
            alt_client = None
            if ns['namespace'].startswith(self.alt_project_id):
                alt_client = self.os_project_alt_admin.resource_types_client
                client = alt_client
            if alt_client is None:
                client = self.os_project_admin.resource_types_client
            resource_name = "rs_type_of_%s" % (ns['namespace'])
            resource_type = client.create_resource_type_association(
                ns['namespace'], name=resource_name)
            resource_types = {'namespace': ns,
                              'resource_type': resource_type}
            namespace_resource_types.append(resource_types)

        return namespace_resource_types

    def assertRSTypeList(self, ns_rs_types, resp):
        actual_rs_types = set(rt['name'] for rt in resp['resource_types'])

        for rs_type in ns_rs_types:
            self.assertIn(rs_type['resource_type']['name'],
                          actual_rs_types)


class MetadefV2RbacResourceTypeTemplate(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def test_create_resource_type(self):
        """Test add_metadef_resource_type_association policy."""
        pass

    @abc.abstractmethod
    def test_get_resource_type(self):
        """Test get_metadef_resource_type policy."""
        pass

    @abc.abstractmethod
    def test_list_resource_types(self):
        """Test list_metadef_resource_types policy."""
        pass

    @abc.abstractmethod
    def test_delete_resource_type(self):
        """Test remove_metadef_resource_type_association policy."""
        pass


class MetadefV2RbacObjectsTest(RbacMetadefBase):
    @classmethod
    def setup_clients(cls):
        super(MetadefV2RbacObjectsTest, cls).setup_clients()
        if 'project_member' in cls.credentials:
            persona = 'project_member'
            alt_persona = 'project_alt_member'
        elif 'project_reader' in cls.credentials:
            persona = 'project_reader'
            alt_persona = 'project_alt_reader'
        else:
            persona = 'project_admin'
            alt_persona = 'project_alt_admin'
        cls.persona = getattr(cls, 'os_%s' % persona)
        cls.alt_persona = getattr(cls, 'os_%s' % alt_persona)
        cls.project_id = cls.persona.namespace_objects_client.project_id
        cls.alt_project_id = \
            cls.alt_persona.namespace_objects_client.project_id
        cls.objects_client = cls.persona.namespace_objects_client

    def create_objects(self):
        # Create namespace for two different projects
        namespaces = self.create_namespaces()

        client = self.os_project_admin.namespace_objects_client
        namespace_objects = []
        for ns in namespaces:
            if ns['namespace'].startswith(self.project_id):
                client = self.os_project_alt_admin.namespace_objects_client
            object_name = "object_of_%s" % (ns['namespace'])
            namespace_object = client.create_namespace_object(
                ns['namespace'], name=object_name,
                description=data_utils.arbitrary_string())

            obj = {'namespace': ns, 'object': namespace_object}
            namespace_objects.append(obj)

        return namespace_objects

    def assertObjectsList(self, actual_obj, client, owner=None):
        ns = actual_obj['namespace']
        if owner:
            if not (ns['visibility'] == 'public' or
                    ns['owner'] == owner):
                self.do_request('list_namespace_objects',
                                expected_status=exceptions.NotFound,
                                namespace=ns['namespace'],
                                client=client)
            else:
                resp = self.do_request('list_namespace_objects',
                                       expected_status=200,
                                       namespace=ns['namespace'],
                                       client=client)
                self.assertIn(actual_obj['object']['name'],
                              resp['objects'][0]['name'])
        else:
            resp = self.do_request('list_namespace_objects',
                                   expected_status=200,
                                   namespace=ns['namespace'],
                                   client=client)
            self.assertIn(actual_obj['object']['name'],
                          resp['objects'][0]['name'])


class MetadefV2RbacObjectsTemplate(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def test_create_object(self):
        """Test add_metadef_object policy."""
        pass

    @abc.abstractmethod
    def test_get_object(self):
        """Test get_metadef_object policy."""
        pass

    @abc.abstractmethod
    def test_list_objects(self):
        """Test list_metadef_objects policy."""
        pass

    @abc.abstractmethod
    def test_update_object(self):
        """Test update_metadef_object policy."""
        pass

    @abc.abstractmethod
    def test_delete_object(self):
        """Test delete_metadef_object policy."""
        pass


class MetadefV2RbacPropertiesTest(RbacMetadefBase):
    @classmethod
    def setup_clients(cls):
        super(MetadefV2RbacPropertiesTest, cls).setup_clients()
        if 'project_member' in cls.credentials:
            persona = 'project_member'
            alt_persona = 'project_alt_member'
        elif 'project_reader' in cls.credentials:
            persona = 'project_reader'
            alt_persona = 'project_alt_reader'
        else:
            persona = 'project_admin'
            alt_persona = 'project_alt_admin'
        cls.persona = getattr(cls, 'os_%s' % persona)
        cls.alt_persona = getattr(cls, 'os_%s' % alt_persona)
        cls.project_id = cls.persona.namespaces_client.project_id
        cls.alt_project_id = cls.alt_persona.namespaces_client.project_id
        cls.properties_client = cls.persona.namespace_properties_client

    def create_properties(self):
        # Create namespace for two different projects
        namespaces = self.create_namespaces()

        namespace_properties = []
        for ns in namespaces:
            alt_client = None
            if ns['namespace'].startswith(self.alt_project_id):
                alt_client = \
                    self.os_project_alt_admin.namespace_properties_client
                client = alt_client
            if alt_client is None:
                client = self.os_project_admin.namespace_properties_client

            property_name = "prop_of_%s" % (ns['namespace'])
            namespace_property = client.create_namespace_property(
                ns['namespace'], name=property_name, title='property',
                type='integer')

            prop = {'namespace': ns, 'property': namespace_property}
            namespace_properties.append(prop)

        return namespace_properties

    def assertPropertyList(self, actual_prop, client, owner=None):
        ns = actual_prop['namespace']
        if owner:
            if not (ns['visibility'] == 'public' or ns['owner'] == owner):
                resp = self.do_request('list_namespace_properties',
                                       expected_status=exceptions.NotFound,
                                       client=client,
                                       namespace=ns['namespace'])
            else:
                resp = self.do_request('list_namespace_properties',
                                       expected_status=200,
                                       client=client,
                                       namespace=ns['namespace'])
                self.assertEqual(actual_prop['property']['name'],
                                 [*resp['properties']][0])
        else:
            resp = self.do_request('list_namespace_properties',
                                   expected_status=200,
                                   client=client,
                                   namespace=ns['namespace'])
            self.assertEqual(actual_prop['property']['name'],
                             [*resp['properties']][0])


class MetadefV2RbacPropertiesTemplate(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def test_create_property(self):
        """Test add_metadef_property policy."""
        pass

    @abc.abstractmethod
    def test_get_properties(self):
        """Test get_metadef_property policy."""
        pass

    @abc.abstractmethod
    def test_list_properties(self):
        """Test list_metadef_properties policy."""
        pass

    @abc.abstractmethod
    def test_update_properties(self):
        """Test update_metadef_property policy."""
        pass

    @abc.abstractmethod
    def test_delete_properties(self):
        """Test delete_metadef_property policy."""
        pass


class MetadefV2RbacTagsTest(RbacMetadefBase):
    @classmethod
    def setup_clients(cls):
        super(MetadefV2RbacTagsTest, cls).setup_clients()
        if 'project_member' in cls.credentials:
            persona = 'project_member'
            alt_persona = 'project_alt_member'
        elif 'project_reader' in cls.credentials:
            persona = 'project_reader'
            alt_persona = 'project_alt_reader'
        else:
            persona = 'project_admin'
            alt_persona = 'project_alt_admin'
        cls.persona = getattr(cls, 'os_%s' % persona)
        cls.alt_persona = getattr(cls, 'os_%s' % alt_persona)
        cls.project_id = cls.persona.namespaces_client.project_id
        cls.alt_project_id = cls.alt_persona.namespaces_client.project_id
        cls.tags_client = cls.persona.namespace_tags_client

    def create_tags(self, namespaces, multiple_tags=False):
        namespace_tags = []

        if multiple_tags:
            tags = [{"name": "tag1"}, {"name": "tag2"}, {"name": "tag3"}]
            for ns in namespaces:
                alt_client = None
                if ns['namespace'].startswith(self.alt_project_id):
                    alt_client = \
                        self.os_project_alt_admin.namespace_tags_client
                    client = alt_client
                if alt_client is None:
                    client = self.os_project_admin.namespace_tags_client
                multiple_tags = client.create_namespace_tags(
                    ns['namespace'], tags=tags)

                namespace_multiple_tags = {'namespace': ns,
                                           'tags': multiple_tags}
                namespace_tags.append(namespace_multiple_tags)
        else:
            for ns in namespaces:
                alt_client = None
                if ns['namespace'].startswith(self.alt_project_id):
                    alt_client = \
                        self.os_project_alt_admin.namespace_tags_client
                    client = alt_client
                if alt_client is None:
                    client = self.os_project_admin.namespace_tags_client
                tag_name = "tag_of_%s" % (ns['namespace'])
                namespace_tag = client.create_namespace_tag(
                    ns['namespace'], tag_name=tag_name)

                tag = {'namespace': ns, 'tag': namespace_tag}
                namespace_tags.append(tag)

        return namespace_tags

    def assertTagsList(self, actual_tag, client, owner=None):
        ns = actual_tag['namespace']
        if owner:
            if not (ns['visibility'] == 'public' or ns['owner'] == owner):
                self.do_request('list_namespace_tags',
                                expected_status=exceptions.NotFound,
                                client=client,
                                namespace=ns['namespace'])
            else:
                resp = self.do_request('list_namespace_tags',
                                       expected_status=200,
                                       client=client,
                                       namespace=ns['namespace'])
                self.assertEqual(actual_tag['tag']['name'],
                                 resp['tags'][0]['name'])
        else:
            resp = self.do_request('list_namespace_tags',
                                   expected_status=200,
                                   client=client,
                                   namespace=ns['namespace'])
            self.assertEqual(actual_tag['tag']['name'],
                             resp['tags'][0]['name'])


class MetadefV2RbacTagsTemplate(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def test_create_tag(self):
        """Test add_metadef_tag policy."""
        pass

    @abc.abstractmethod
    def test_get_tags(self):
        """Test get_metadef_tag policy."""
        pass

    @abc.abstractmethod
    def test_list_tags(self):
        """Test list_metadef_tags policy."""
        pass

    @abc.abstractmethod
    def test_update_tags(self):
        """Test update_metadef_tag policy."""
        pass

    @abc.abstractmethod
    def test_delete_tags(self):
        """Test delete_metadef_tag policy."""
        pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

glance_api_group = cfg.OptGroup(name="glance_api",
                                title="Glance API Service Options")

GlanceAPIGroup = [cfg.BoolOpt("system_scope_supported",
                              default=False,
                              help="Whether or not system scope "
                                   "is supported")]
